/* Name: main.c
 * Author: <insert your name here>
 * Copyright: <insert your copyright message here>
 * License: <insert your license reference here>
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <util/twi.h>
#include "USI_TWI_Master.h"

#define SLAVE_ADDR  0x10

#define MESSAGEBUF_SIZE       10

#define TWI_GEN_CALL         0x00  // The General Call address is 0

// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20

// Sample TWI transmission states, used in the main application.
#define SEND_DATA             0x01
#define REQUEST_DATA          0x02
#define READ_DATA_FROM_BUFFER 0x03

volatile uint8_t clockStarted = 0;
volatile uint32_t sleepcount = 0;
void displayValue(uint8_t val) {

    unsigned char messageBuf[MESSAGEBUF_SIZE];
    unsigned char TWI_targetSlaveAddress, temp, pressedButton, myCounter=0;
    TWI_targetSlaveAddress   = 0x20;
    messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
    messageBuf[1] = 0xA2;             // The first byte is used for commands.
    messageBuf[2] = (val<29) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[3] = (val<39) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[4] = (val<50) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[5] = (val<75) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[6] = (val<90) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[7] = (val<110) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[8] = (val<130) ? 255 : 0;                        // The second byte is used for the data.
    messageBuf[9] = (val<150) ? 255 : 0;                        // The second byte is used for the data.
    temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 10 );
    _delay_ms(1);
}

void enterSleepMode(uint8_t initialValue) {
    displayValue(250);
    uint8_t delta=0;
    
    while (delta<10) {
        _delay_ms(10000);
        PORTB &= (~(0x04));
        DDRB |= 0X04; //Enable ping output
        PORTB |= 0X04;
        _delay_us(5);
        PORTB &= (~(0x04));
        DDRB &= (~(0X04));
        
        
        clockStarted = 0;
        MCUCR = (1<<ISC01)|(1<<ISC00);    //Rising edge of INT0 generates interrupt request
        GIMSK = (1<<INT0);   //external interrupt 0 enabled
        
        TCCR1B = 0x03; //clk/? prescale
        
        while(!clockStarted);
        MCUCR = (1<<ISC01)|(0<<ISC00);  //Change to falling edge
        while(clockStarted);
        GIMSK = (0<<INT0);   //external interrupt 0 disabled
        
        if (initialValue > TCNT1L)
            delta = initialValue-TCNT1L;
        else
            delta = TCNT1L-initialValue;
        
    }
    sleepcount = 0;
}

int main(void)
{
    _delay_ms(100);
    DDRA |= 0X02; //LED RESET LINE
    PORTA &= ~(0x02);
    _delay_ms(10);
    PORTA |= (0X02);
    
    unsigned char messageBuf[MESSAGEBUF_SIZE];
    unsigned char TWI_targetSlaveAddress, temp, pressedButton, myCounter=0;
    
    
    USI_TWI_Master_Initialise();//seriously Atmel? no spellcheck?
    sei();
    TWI_targetSlaveAddress   = 0x20;
    
    //enable internal clock
    messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
    messageBuf[1] = 0x00;             // The first byte is used for commands.
    messageBuf[2] = 0x00;                        // The second byte is used for the data.
    temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );

    //LED output state 0
    messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
    messageBuf[1] = 0x0C;             // The first byte is used for commands.
    messageBuf[2] = 0xAA;                        // The second byte is used for the data.
    temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );
    
    //LED output state 1
    messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
    messageBuf[1] = 0x0D;             // The first byte is used for commands.
    messageBuf[2] = 0xAA;                        // The second byte is used for the data.
    temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );
    
    

    
    uint8_t i;
    uint8_t outputBuffer[8] = {0,0,0,0,0,0,0,0};
    for (i=0;i<8;i++) {
        uint8_t j;
        for (j=0;j<250;j+=5) {
            outputBuffer[7-i]=j;
            
            messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
            messageBuf[1] = 0xA2;             // The first byte is used for commands.
            messageBuf[2] = outputBuffer[0];                        // The second byte is used for the data.
            messageBuf[3] = outputBuffer[1];                        // The second byte is used for the data.
            messageBuf[4] = outputBuffer[2];                        // The second byte is used for the data.
            messageBuf[5] = outputBuffer[3];                        // The second byte is used for the data.
            messageBuf[6] = outputBuffer[4];                        // The second byte is used for the data.
            messageBuf[7] = outputBuffer[5];                        // The second byte is used for the data.
            messageBuf[8] = outputBuffer[6];                        // The second byte is used for the data.
            messageBuf[9] = outputBuffer[7];                        // The second byte is used for the data.
            temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 10 );
            _delay_ms(1);

        }
    }
    
    while(1) {
        
    
    _delay_ms(100);
    PORTB &= (~(0x04));
    DDRB |= 0X04; //Enable ping output
    PORTB |= 0X04;
    _delay_us(5);
    PORTB &= (~(0x04));
    DDRB &= (~(0X04));
    
    
    clockStarted = 0;
    MCUCR = (1<<ISC01)|(1<<ISC00);    //Rising edge of INT0 generates interrupt request
    GIMSK = (1<<INT0);   //external interrupt 0 enabled
 
    TCCR1B = 0x03; //clk/? prescale
    
    while(!clockStarted);
    MCUCR = (1<<ISC01)|(0<<ISC00);  //Change to falling edge
    while(clockStarted);
    GIMSK = (0<<INT0);   //external interrupt 0 disabled
    
        if (TCNT1H>0)
            displayValue(255);
        else
            displayValue(TCNT1L);
        if ((TCNT1L<29)||(TCNT1L>=150)) {
            sleepcount++;
        }
        else {
            sleepcount=0;
        }
        _delay_ms(1000);
        if (sleepcount>= 100)
            enterSleepMode(TCNT1L);
    
}

}


ISR (EXT_INT0_vect)
{



    if (!clockStarted) {
        TCNT1H = 0;
        TCNT1L = 0;
        clockStarted = 1;
    }
    else
    {
        TCCR1B = 0;
        clockStarted = 0;
    }
}